@@ -11,11 +11,19 @@
 #' @export getCSMF
 #'
 #' @examples
+#' \dontrun{
+#' library(InSilicoVA)
 #' data(RandomVA1)
 #' # for illustration, only use interVA on 100 deaths
-#' fit <- codeVA(RandomVA1[1:100, ], data.type = "WHO", model = "InterVA", 
-#'                   version = "4.02", HIV = "h", Malaria = "l")
+#' fit <- codeVA(RandomVA1[1:100, ], data.type = "WHO2012", model = "InterVA", 
+#'                   version = "4.03", HIV = "h", Malaria = "l")
 #' getCSMF(fit)
+#' library(InterVA5)
+#' data(RandomVA5)
+#' fit <- codeVA(RandomVA5[1:100, ], data.type = "WHO2016", model = "InterVA", 
+#'                   version = "5.0", HIV = "h", Malaria = "l")
+#' getCSMF(fit)
+#' }
 #' 
 getCSMF <- function(x, CI = 0.95, interVA.rule = TRUE){
 
@@ -27,6 +35,9 @@ getCSMF <- function(x, CI = 0.95, interVA.rule = TRUE){
   if(class(x) == "interVA"){
     return(CSMF(x, InterVA.rule = interVA.rule, noplot = TRUE))
   } 
+  if(class(x) == "interVA5"){
+    return(CSMF5(x, InterVA.rule = interVA.rule, noplot = TRUE))
+  } 
    
   if(class(x) == "tariff"){
     return(x$csmf)
@@ -109,7 +120,7 @@ getTopCOD <- function(x, interVA.rule = TRUE){
     probs <- x$indiv.prob
     pick <- colnames(probs)[apply(probs, 1, which.max)]
     id <- x$id
-  }else if(class(x) == "interVA"){
+  }else if(class(x) == "interVA" ){
       id <- x$ID
       pick <- rep("", length(x$VA))
       for(i in 1:length(x$VA)){
@@ -129,6 +140,26 @@ getTopCOD <- function(x, interVA.rule = TRUE){
           }
       }
       pick[which(pick == " ")] <- "Undetermined"
+    }else if(class(x) == "interVA5"){
+      id <- x$ID
+      pick <- rep("", length(x$VA))
+      for(i in 1:length(x$VA)){
+        if(interVA.rule){
+          pick[i] <- x$VA[[i]]$CAUSE1
+        }else{
+            prob <- x$VA[[i]]$wholeprob[4:64]
+            causenames <- names(prob)
+            causeindex <- 1:length(causenames)
+            if(causenames[1] == "Not pregnant or recently delivered" &&
+                causenames[2] == "Pregnancy ended within 6 weeks of death" &&
+                causenames[3] == "Pregnant at death"){
+                    causeindex <- causeindex[-c(1:3)]
+                    causenames <- causenames[-c(1:3)]    
+            }
+            pick[i] <- causenames[which.max(prob[causeindex])]
+          }
+      }
+      pick[which(pick == " ")] <- "Undetermined"
     }else if(class(x) == "tariff"){
       pick <- x$causes.test[, 2]
       id <- as.character(x$causes.test[, 1])
@@ -146,7 +177,7 @@ getTopCOD <- function(x, interVA.rule = TRUE){
 #' @param CI Credible interval for posterior estimates. If CI is set to TRUE, a list is returned instead of a data frame.
 #'
 #' @return a data frame of COD distribution for each individual specified by row names.
-#' @export getTopCOD
+#' @export getIndivProb
 #'
 #' @examples
 #' data(RandomVA1)
@@ -179,6 +210,15 @@ getIndivProb <- function(x, CI = NULL){
       rownames(probs) <- id
       colnames(probs) <- names(x$VA[[i]]$wholeprob)
 
+    }else if(class(x) == "interVA5"){
+      id <- x$ID
+      probs <- matrix(NA, length(x$VA), length(x$VA[[1]]$wholeprob[4:64]))
+      for(i in 1:length(x$VA)){
+          probs[i, ] <- x$VA[[i]]$wholeprob[4:64]
+      }
+      rownames(probs) <- id
+      colnames(probs) <- names(x$VA[[i]]$wholeprob[4:64])
+
     }else if(class(x) == "tariff"){
       warning("Tariff method produces only rankings of causes, not probabilities")
       probs <- x$score
